@@ -11,13 +11,14 @@ from typing import Optional, Generator, Union, Any
 import pytz
 import httpx
 
+from .engines import DatabaseEngine, Atomic
 from .models import ModelBase, MODEL
 from .utils import parse_tsv, import_submodules
 from .session import ctx_session_id, ctx_session_timeout
 
 
-logger = logging.getLogger('clickhouse_orm')
-Page = namedtuple('Page', 'objects number_of_objects pages_total number page_size')
+logger = logging.getLogger("clickhouse_orm")
+Page = namedtuple("Page", "objects number_of_objects pages_total number page_size")
 
 
 class DatabaseException(Exception):
@@ -30,6 +31,7 @@ class ServerError(DatabaseException):
     """
     Raised when a server returns an error.
     """
+
     def __init__(self, message):
         self.code = None
         processed = self.get_error_code_msg(message)
@@ -43,21 +45,30 @@ class ServerError(DatabaseException):
 
     ERROR_PATTERNS = (
         # ClickHouse prior to v19.3.3
-        re.compile(r'''
+        re.compile(
+            r"""
             Code:\ (?P<code>\d+),
             \ e\.displayText\(\)\ =\ (?P<type1>[^ \n]+):\ (?P<msg>.+?),
             \ e.what\(\)\ =\ (?P<type2>[^ \n]+)
-        ''', re.VERBOSE | re.DOTALL),
+        """,
+            re.VERBOSE | re.DOTALL,
+        ),
         # ClickHouse v19.3.3+
-        re.compile(r'''
+        re.compile(
+            r"""
             Code:\ (?P<code>\d+),
             \ e\.displayText\(\)\ =\ (?P<type1>[^ \n]+):\ (?P<msg>.+)
-        ''', re.VERBOSE | re.DOTALL),
+        """,
+            re.VERBOSE | re.DOTALL,
+        ),
         # ClickHouse v21+
-        re.compile(r'''
+        re.compile(
+            r"""
             Code:\ (?P<code>\d+).
             \ (?P<type1>[^ \n]+):\ (?P<msg>.+)
-        ''', re.VERBOSE | re.DOTALL),
+        """,
+            re.VERBOSE | re.DOTALL,
+        ),
     )
 
     @classmethod
@@ -72,7 +83,7 @@ class ServerError(DatabaseException):
             match = pattern.match(full_error_message)
             if match:
                 # assert match.group('type1') == match.group('type2')
-                return int(match.group('code')), match.group('msg').strip()
+                return int(match.group("code")), match.group("msg").strip()
 
         return 0, full_error_message
 
@@ -86,11 +97,22 @@ class Database:
     Database instances connect to a specific ClickHouse database for running queries,
     inserting data and other operations.
     """
+
     _client_class = httpx.Client
 
-    def __init__(self, db_name, db_url='http://localhost:8123/',
-                 username=None, password=None, readonly=False, auto_create=True,
-                 timeout=60, verify_ssl_cert=True, log_statements=False):
+    def __init__(
+        self,
+        db_name,
+        db_url="http://localhost:8123/",
+        username=None,
+        password=None,
+        readonly=False,
+        auto_create=True,
+        timeout=60,
+        verify_ssl_cert=True,
+        log_statements=False,
+        engine: DatabaseEngine = Atomic(),
+    ):
         """
         Initializes a database instance. Unless it's readonly, the database will be
         created on the ClickHouse server if it does not already exist.
@@ -105,6 +127,7 @@ class Database:
         - `timeout`: the connection timeout in seconds.
         - `verify_ssl_cert`: whether to verify the server's certificate when connecting via HTTPS.
         - `log_statements`: when True, all database statements are logged.
+        - `engine`: By default, ClickHouse uses the Atomic database engine.
         """
         self.db_name = db_name
         self.db_url = db_url
@@ -112,9 +135,10 @@ class Database:
         self._readonly = readonly
         self.auto_create = auto_create
         self.timeout = timeout
+        self.engine = engine
         self.request_session = self._client_class(verify=verify_ssl_cert, timeout=timeout)
         if username:
-            self.request_session.auth = (username, password or '')
+            self.request_session.auth = (username, password or "")
         self.log_statements = log_statements
         self.settings = {}
         self.db_exists = False  # this is required before running _is_existing_database
@@ -134,7 +158,7 @@ class Database:
         if self._readonly:
             if not self.db_exists:
                 raise DatabaseException(
-                    'Database does not exist, and cannot be created under readonly connection'
+                    "Database does not exist, and cannot be created under readonly connection"
                 )
             self.connection_readonly = self._is_connection_readonly()
             self.readonly = True
@@ -155,14 +179,17 @@ class Database:
         """
         Creates the database on the ClickHouse server if it does not already exist.
         """
-        self._send('CREATE DATABASE IF NOT EXISTS `%s`' % self.db_name)
+        self._send(
+            "CREATE DATABASE IF NOT EXISTS `%s` ENGINE = %s"
+            % (self.db_name, self.engine.create_database_sql())
+        )
         self.db_exists = True
 
     def drop_database(self):
         """
         Deletes the database on the ClickHouse server.
         """
-        self._send('DROP DATABASE `%s`' % self.db_name)
+        self._send("DROP DATABASE `%s`" % self.db_name)
         self.db_exists = False
 
     def create_table(self, model_class: type[MODEL]) -> None:
@@ -171,7 +198,7 @@ class Database:
         """
         if model_class.is_system_model():
             raise DatabaseException("You can't create system table")
-        if getattr(model_class, 'engine') is None:
+        if getattr(model_class, "engine") is None:
             raise DatabaseException("%s class must define an engine" % model_class.__name__)
         self._send(model_class.create_table_sql(self))
 
@@ -190,13 +217,9 @@ class Database:
         """
         sql = "SELECT count() FROM system.tables WHERE database = '%s' AND name = '%s'"
         r = self._send(sql % (self.db_name, model_class.table_name()))
-        return r.text.strip() == '1'
+        return r.text.strip() == "1"
 
-    def get_model_for_table(
-        self,
-        table_name: str,
-        system_table: bool = False
-    ):
+    def get_model_for_table(self, table_name: str, system_table: bool = False):
         """
         Generates a model class from an existing table in the database.
         This can be used for querying tables which don't have a corresponding model class,
@@ -205,7 +228,7 @@ class Database:
         - `table_name`: the table to create a model for
         - `system_table`: whether the table is a system table, or belongs to the current database
         """
-        db_name = 'system' if system_table else self.db_name
+        db_name = "system" if system_table else self.db_name
         sql = "DESCRIBE `%s`.`%s` FORMAT TSV" % (db_name, table_name)
         lines = self._send(sql).iter_lines()
         fields = [parse_tsv(line)[:2] for line in lines]
@@ -222,7 +245,7 @@ class Database:
         The name must be string, and the value is converted to string in case
         it isn't. To remove a setting, pass `None` as the value.
         """
-        assert isinstance(name, str), 'Setting name must be a string'
+        assert isinstance(name, str), "Setting name must be a string"
         if value is None:
             self.settings.pop(name, None)
         else:
@@ -246,14 +269,13 @@ class Database:
         if first_instance.is_read_only() or first_instance.is_system_model():
             raise DatabaseException("You can't insert into read only and system tables")
 
-        fields_list = ','.join(
-            ['`%s`' % name for name in first_instance.fields(writable=True)])
-        fmt = 'TSKV' if model_class.has_funcs_as_defaults() else 'TabSeparated'
-        query = 'INSERT INTO $table (%s) FORMAT %s\n' % (fields_list, fmt)
+        fields_list = ",".join(["`%s`" % name for name in first_instance.fields(writable=True)])
+        fmt = "TSKV" if model_class.has_funcs_as_defaults() else "TabSeparated"
+        query = "INSERT INTO $table (%s) FORMAT %s\n" % (fields_list, fmt)
 
         def gen():
             buf = BytesIO()
-            buf.write(self._substitute(query, model_class).encode('utf-8'))
+            buf.write(self._substitute(query, model_class).encode("utf-8"))
             first_instance.set_database(self)
             buf.write(first_instance.to_db_string())
             # Collect lines in batches of batch_size
@@ -271,12 +293,11 @@ class Database:
             # Return any remaining lines in partial batch
             if lines:
                 yield buf.getvalue()
+
         self._send(gen())
 
     def count(
-        self,
-        model_class: Optional[type[MODEL]],
-        conditions: Optional[Union[str, 'Q']] = None
+        self, model_class: Optional[type[MODEL]], conditions: Optional[Union[str, "Q"]] = None
     ) -> int:
         """
         Counts the number of records in the model's table.
@@ -286,20 +307,17 @@ class Database:
         """
         from clickhouse_orm.query import Q
 
-        query = 'SELECT count() FROM $table'
+        query = "SELECT count() FROM $table"
         if conditions:
             if isinstance(conditions, Q):
                 conditions = conditions.to_sql(model_class)
-            query += ' WHERE ' + str(conditions)
+            query += " WHERE " + str(conditions)
         query = self._substitute(query, model_class)
         r = self._send(query)
         return int(r.text) if r.text else 0
 
     def select(
-        self,
-        query: str,
-        model_class: Optional[type[MODEL]] = None,
-        settings: Optional[dict] = None
+        self, query: str, model_class: Optional[type[MODEL]] = None, settings: Optional[dict] = None
     ) -> Generator[MODEL, None, None]:
         """
         Performs a query and returns a generator of model instances.
@@ -309,7 +327,7 @@ class Database:
           or `None` for getting back instances of an ad-hoc model.
         - `settings`: query settings to send as HTTP GET parameters
         """
-        query += ' FORMAT TabSeparatedWithNamesAndTypes'
+        query += " FORMAT TabSeparatedWithNamesAndTypes"
         query = self._substitute(query, model_class)
         r = self._send(query, settings, True)
         try:
@@ -345,7 +363,7 @@ class Database:
         page_num: int = 1,
         page_size: int = 100,
         conditions=None,
-        settings: Optional[dict] = None
+        settings: Optional[dict] = None,
     ):
         """
         Selects records and returns a single page of model instances.
@@ -362,27 +380,28 @@ class Database:
         `pages_total`, `number` (of the current page), and `page_size`.
         """
         from clickhouse_orm.query import Q
+
         count = self.count(model_class, conditions)
         pages_total = int(ceil(count / float(page_size)))
         if page_num == -1:
             page_num = max(pages_total, 1)
         elif page_num < 1:
-            raise ValueError('Invalid page number: %d' % page_num)
+            raise ValueError("Invalid page number: %d" % page_num)
         offset = (page_num - 1) * page_size
-        query = 'SELECT * FROM $table'
+        query = "SELECT * FROM $table"
         if conditions:
             if isinstance(conditions, Q):
                 conditions = conditions.to_sql(model_class)
-            query += ' WHERE ' + str(conditions)
-        query += ' ORDER BY %s' % order_by
-        query += ' LIMIT %d, %d' % (offset, page_size)
+            query += " WHERE " + str(conditions)
+        query += " ORDER BY %s" % order_by
+        query += " LIMIT %d, %d" % (offset, page_size)
         query = self._substitute(query, model_class)
         return Page(
             objects=list(self.select(query, model_class, settings)) if count else [],
             number_of_objects=count,
             pages_total=pages_total,
             number=page_num,
-            page_size=page_size
+            page_size=page_size,
         )
 
     def migrate(self, migrations_package_name, up_to=9999):
@@ -395,19 +414,23 @@ class Database:
         """
         from .migrations import MigrationHistory  # pylint: disable=C0415
 
-        logger = logging.getLogger('migrations')
+        logger = logging.getLogger("migrations")
         applied_migrations = self._get_applied_migrations(migrations_package_name)
         modules = import_submodules(migrations_package_name)
         unapplied_migrations = set(modules.keys()) - applied_migrations
         for name in sorted(unapplied_migrations):
-            logger.info('Applying migration %s...', name)
+            logger.info("Applying migration %s...", name)
             for operation in modules[name].operations:
                 operation.apply(self)
-            self.insert([MigrationHistory(
-                package_name=migrations_package_name,
-                module_name=name,
-                applied=datetime.date.today())
-            ])
+            self.insert(
+                [
+                    MigrationHistory(
+                        package_name=migrations_package_name,
+                        module_name=name,
+                        applied=datetime.date.today(),
+                    )
+                ]
+            )
             if int(name[:4]) >= up_to:
                 break
 
@@ -432,19 +455,14 @@ class Database:
         query = self._substitute(query, MigrationHistory)
         return set(obj.module_name for obj in self.select(query))
 
-    def _send(
-        self,
-        data: str | bytes | Generator,
-        settings: dict = None,
-        stream: bool = False
-    ):
+    def _send(self, data: str | bytes | Generator, settings: dict = None, stream: bool = False):
         if isinstance(data, str):
-            data = data.encode('utf-8')
+            data = data.encode("utf-8")
             if self.log_statements:
                 logger.info(data)
         params = self._build_params(settings)
         request = self.request_session.build_request(
-            method='POST', url=self.db_url, content=data, params=params
+            method="POST", url=self.db_url, content=data, params=params
         )
         r = self.request_session.send(request, stream=stream)
         if isinstance(r, httpx.Response) and r.status_code != 200:
@@ -457,52 +475,52 @@ class Database:
         params.update(self.settings)
         params.update(self._context_params)
         if self.db_exists:
-            params['database'] = self.db_name
+            params["database"] = self.db_name
         # Send the readonly flag, unless the connection is already readonly (to prevent db error)
         if self.readonly and not self.connection_readonly:
-            params['readonly'] = '1'
+            params["readonly"] = "1"
         return params
 
     def _substitute(self, query, model_class=None):
         """
         Replaces $db and $table placeholders in the query.
         """
-        if '$' in query:
+        if "$" in query:
             mapping = dict(db="`%s`" % self.db_name)
             if model_class:
                 if model_class.is_system_model():
-                    mapping['table'] = "`system`.`%s`" % model_class.table_name()
+                    mapping["table"] = "`system`.`%s`" % model_class.table_name()
                 elif model_class.is_temporary_model():
-                    mapping['table'] = "`%s`" % model_class.table_name()
+                    mapping["table"] = "`%s`" % model_class.table_name()
                 else:
-                    mapping['table'] = "`%s`.`%s`" % (self.db_name, model_class.table_name())
+                    mapping["table"] = "`%s`.`%s`" % (self.db_name, model_class.table_name())
             query = Template(query).safe_substitute(mapping)
         return query
 
     def _get_server_timezone(self):
         try:
-            r = self._send('SELECT timezone()')
+            r = self._send("SELECT timezone()")
             return pytz.timezone(r.text.strip())
         except ServerError as err:
-            logger.exception('Cannot determine server timezone (%s), assuming UTC', err)
+            logger.exception("Cannot determine server timezone (%s), assuming UTC", err)
             return pytz.utc
 
     def _get_server_version(self, as_tuple=True):
         try:
-            r = self._send('SELECT version();')
+            r = self._send("SELECT version();")
             ver = r.text
         except ServerError as err:
-            logger.exception('Cannot determine server version (%s), assuming 1.1.0', err)
-            ver = '1.1.0'
-        return tuple(int(n) for n in ver.split('.') if n.isdigit()) if as_tuple else ver
+            logger.exception("Cannot determine server version (%s), assuming 1.1.0", err)
+            ver = "1.1.0"
+        return tuple(int(n) for n in ver.split(".") if n.isdigit()) if as_tuple else ver
 
     def _is_existing_database(self):
         r = self._send("SELECT count() FROM system.databases WHERE name = '%s'" % self.db_name)
-        return r.text.strip() == '1'
+        return r.text.strip() == "1"
 
     def _is_connection_readonly(self):
         r = self._send("SELECT value FROM system.settings WHERE name = 'readonly'")
-        return r.text.strip() != '0'
+        return r.text.strip() != "0"
 
 
 # Expose only relevant classes in import *
